import os
from opendatasets.utils.network import urlopen, download_url, is_url
from opendatasets.utils.archive import extract_archive


def is_kaggle_url(url):
    prefixes = ['kaggle.com',
                'www.kaggle.com',
                'http://kaggle.com',
                'http://www.kaggle.com',
                'https://www.kaggle.com',
                'https://kaggle.com']
    return any((url.startswith(prefix) for prefix in prefixes))


def get_kaggle_dataset_id(dataset_id_or_url):
    parts = []
    if is_kaggle_url(dataset_id_or_url):
        parts = dataset_id_or_url.split('kaggle.com/')[1].split('/')[:2]
    elif not is_url(dataset_id_or_url):
        parts = dataset_id_or_url.split('/')[:2]
    assert len(parts) == 2, 'Invalid Kaggle dataset URL or ID: ' + \
        dataset_id_or_url
    return '/'.join(parts)


def get_kaggle_download_hash(dataset_id):
    download_page_url = 'https://www.kaggle.com/' + dataset_id + '/download'
    with urlopen(download_page_url) as response:
        redirect_url = response.geturl()
    download_hash = redirect_url.split('downloadHash%3D')[1].split('&')[0]
    return download_hash


def download_kaggle_dataset(dataset_url, data_dir='.', force=True, dry_run=False):
    dataset_id = get_kaggle_dataset_id(dataset_url)
    print('Kaggle dataset ID: ', dataset_id)
    raw_dataset_url = ('https://www.kaggle.com/' +
                       dataset_id +
                       '/download?resource=download&downloadHash=' +
                       get_kaggle_download_hash(dataset_id))
    folder_name = dataset_id.split('/')[-1]
    archive_name = folder_name + '.zip'
    download_url(raw_dataset_url, root=data_dir,
                 filename=archive_name, force=force, dry_run=dry_run)

    extract_archive(os.path.join(data_dir, archive_name),
                    os.path.join(data_dir, folder_name))



import os
from opendatasets.utils.md5 import check_integrity
from tqdm import tqdm


def _quota_exceeded(response):
    return "Google Drive - Quota exceeded" in response.text


def download_file_from_google_drive(file_id, root, filename=None, md5=None):
    """Download a Google Drive file from  and place it in root.
    Args:
        file_id (str): id of file to be downloaded
        root (str): Directory to place downloaded file in
        filename (str, optional): Name to save the file under. If None, use the id of the file.
        md5 (str, optional): MD5 checksum of the download. If None, do not check
    """
    # Based on https://stackoverflow.com/questions/38511444/python-download-files-from-google-drive-using-url
    try:
        import requests
    except ImportError as e:
        print(
            'To download from Google Drive, first install the requests library by running:')
        print('pip install requests --upgrade')
        raise e
    url = "https://docs.google.com/uc?export=download"

    root = os.path.expanduser(root)
    if not filename:
        filename = file_id
    fpath = os.path.join(root, filename)

    os.makedirs(root, exist_ok=True)

    if os.path.isfile(fpath) and check_integrity(fpath, md5):
        print('Using downloaded and verified file: ' + fpath)
    else:
        session = requests.Session()

        response = session.get(url, params={'id': file_id}, stream=True)
        token = _get_confirm_token(response)

        if token:
            params = {'id': file_id, 'confirm': token}
            response = session.get(url, params=params, stream=True)

        if _quota_exceeded(response):
            msg = (
                "The daily quota of the file " + filename + " is exceeded and it " +
                "can't be downloaded. This is a limitation of Google Drive " +
                "and can only be overcome by trying again later."
            )
            raise RuntimeError(msg)

        _save_response_content(response, fpath)


def _get_confirm_token(response):
    for key, value in response.cookies.items():
        if key.startswith('download_warning'):
            return value

    return None


def _save_response_content(response, destination, chunk_size=32768):
    with open(destination, "wb") as f:
        pbar = tqdm(total=None)
        progress = 0
        for chunk in response.iter_content(chunk_size):
            if chunk:  # filter out keep-alive new chunks
                f.write(chunk)
                progress += len(chunk)
                pbar.update(progress - pbar.n)
        pbar.close()



import os
import tarfile
import zipfile
import gzip


def _is_tarxz(filename):
    return filename.endswith(".tar.xz")


def _is_tar(filename):
    return filename.endswith(".tar")


def _is_targz(filename):
    return filename.endswith(".tar.gz")


def _is_tgz(filename):
    return filename.endswith(".tgz")


def _is_gzip(filename):
    return filename.endswith(".gz") and not filename.endswith(".tar.gz")


def _is_zip(filename):
    return filename.endswith(".zip")


def extract_archive(from_path, to_path=None, remove_finished=False, dry_run=False):
    print("Extracting archive " + from_path + " to " + to_path)
    if dry_run:
        print("This is a dry run, skipping...")
        return
    if to_path is None:
        to_path = os.path.dirname(from_path)

    if _is_tar(from_path):
        with tarfile.open(from_path, 'r') as tar:
            tar.extractall(path=to_path)
    elif _is_targz(from_path) or _is_tgz(from_path):
        with tarfile.open(from_path, 'r:gz') as tar:
            tar.extractall(path=to_path)
    elif _is_tarxz(from_path):
        with tarfile.open(from_path, 'r:xz') as tar:
            tar.extractall(path=to_path)
    elif _is_gzip(from_path):
        to_path = os.path.join(to_path, os.path.splitext(
            os.path.basename(from_path))[0])
        with open(to_path, "wb") as out_f, gzip.GzipFile(from_path) as zip_f:
            out_f.write(zip_f.read())
    elif _is_zip(from_path):
        with zipfile.ZipFile(from_path, 'r') as z:
            z.extractall(to_path)
    else:
        raise ValueError("Extraction of {} not supported".format(from_path))

    if remove_finished:
        os.remove(from_path)



from opendatasets.utils import download_raw_files


def download(dataset_id, data_dir, dry_run, **kwargs):
    download_raw_files(dataset_id,
                       data_dir,
                       dry_run,
                       ['state-of-javascript-2016-responses.ndjson'])



from opendatasets.utils import download_raw_files


def download(dataset_id, data_dir, dry_run, **kwargs):
    download_raw_files(dataset_id, data_dir, dry_run, [
        'survey_results_public.csv',
        'survey_results_schema.csv',
        'README.txt'
    ])



import os
import urllib
import re
from tqdm import tqdm
from opendatasets.utils.md5 import check_integrity

try:
    urlopen = urllib.request.urlopen
except Exception:
    # For Python 2.7
    urlopen = urllib.urlopen

try:
    urlretrieve = urllib.request.urlretrieve
except Exception:
    # For Python 2.7
    urlretrieve = urllib.urlretrieve


def download_url(url, root, filename=None, md5=None, force=False, dry_run=False):
    """Download a file from a url and place it in root.
    Args:
        url (str): URL to download file from
        root (str): Directory to place downloaded file in
        filename (str, optional): 
        Name to save the file under. If None, use the basename of the URL
        md5 (str, optional): MD5 checksum of the download. If None, do not check
    """
    root = os.path.expanduser(root)
    if not filename:
        filename = os.path.basename(url)
    fpath = os.path.join(root, filename)

    if not(os.path.exists(root)):
        os.makedirs(root)

    # check if file is already present locally
    if not force and check_integrity(fpath, md5):
        print('Using downloaded and verified file: ' + fpath)
    else:   # download the file
        try:
            print('Downloading ' + url + ' to ' + fpath)
            if not dry_run:
                urlretrieve(
                    url, fpath,
                    reporthook=gen_bar_updater()
                )
        # type: ignore[attr-defined]
        except (urllib.error.URLError, IOError) as e:
            if url[:5] == 'https':
                url = url.replace('https:', 'http:')
                print('Failed download. Trying https -> http instead.'
                      ' Downloading ' + url + ' to ' + fpath)
                urlretrieve(
                    url, fpath,
                    reporthook=gen_bar_updater()
                )
            else:
                raise e
        # check integrity of downloaded file
        if not dry_run and not check_integrity(fpath, md5):
            raise RuntimeError("File not found or corrupted.")


def gen_bar_updater():
    pbar = tqdm(total=None)

    def bar_update(count, block_size, total_size):
        if pbar.total is None and total_size:
            pbar.total = total_size
        progress_bytes = count * block_size
        pbar.update(progress_bytes - pbar.n)

    return bar_update


URL_REGEX = re.compile(
    r'^(?:http|ftp)s?://'  # http:// or https://
    # domain...
    r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|'
    r'localhost|'  # localhost...
    r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})'  # ...or ip
    r'(?::\d+)?'  # optional port
    r'(?:/?|[/?]\S+)$', re.IGNORECASE)


def is_url(url):
    return re.match(URL_REGEX, url) is not None
